@@ -10,45 +10,16 @@ import os,glob,copy
 import json
 
 class Background:
-    def __init__(self,objname='None',gfile='None',tfile='None',extnum=1,
-                 savefolder='EXTRA',overwrite=False,loadinfo=False,
+    def __init__(self,container,objname='None',gfile=None,tfile='None',
                  automask=True,
-                 padxleft=5,padxright=5,
-                 padyup=5,halfdyup=5,halfdylow=5,padylow=5,
-                 adjustx=0,adjusty=0,
+                 params_bbox = {'padxleft':15,'padxright':15,'padyup':15,'halfdyup':20,'halfdylow':20,'padylow':15,'adjustx':0,'adjusty':0},
                  rescale=(False,None,None,None),
                  sigclip=(False,None,None)
                 ):
-        # load instead of instantiate
-        if loadinfo:
-            self.background = self.loadinfo(savefolder)
-            return 
-        # remove/create savefolder
-        if overwrite:
-            os.system('rm -r {0}'.format(savefolder))
-            print('Remove {0}'.format(savefolder))
-        else:
-            tmp = os.getcwd()
-            try:
-                os.chdir(savefolder)
-                os.chdir(tmp)
-                print('Stop: Savefolder {0} already exists, overwrite = {1}'.format(savefolder,overwrite))
-                return
-            except:
-                pass
-        tmp = os.getcwd()
-        for i in savefolder.split('/'):
-            os.system('mkdir {0}'.format(i))
-            os.chdir(i)
-        os.chdir(tmp)            
-        print('Set savefolder {0}'.format(savefolder))
-        # start instantiation
+        self.container = container
         self.data = {'objname':objname,
                      'gfile':gfile,
                      'tfile':tfile,
-                     'extnum':extnum,
-                     'ROOTNAME':'None',
-                     'savefolder':savefolder,
                      'automask':automask,
                      'rescale':rescale[0],
                      'rescale_x1':rescale[1],
@@ -58,19 +29,14 @@ class Background:
                      'sigclip_niter':sigclip[1],
                      'sigclip_sigma':sigclip[2]
                     }
-        self.data['ROOTNAME'] = self._rootname()
         self.trace = self._trace()
-        self.bbox = {'padxleft':padxleft,'padxright':padxright,
-                     'padyup':padyup,'halfdyup':halfdyup,'halfdylow':halfdylow,'padylow':padylow,
-                     'adjustx':adjustx,'adjusty':adjusty,
-                     'bbox':None
-                     }
+        self.bbox = params_bbox
         self.bbox['bbox'] = self._bbox()
         self.bkg = self._bkg()
     def _bkg(self):
         rescale = (self.data['rescale'],self.data['rescale_x1'],self.data['rescale_x2'],self.data['rescale_y'])
         sigclip = (self.data['sigclip'],self.data['sigclip_niter'],self.data['sigclip_sigma'])
-        tmpdata = fits.open(self.data['gfile'])[self.data['extnum']].data
+        tmpdata = fits.open(self.data['gfile'][0])[self.data['gfile'][1]].data
         bb0x,bb1x = self.bbox['bbox']['BBX']
         bb0y,bb1y = self.bbox['bbox']['BBY']
         xg,yg = self.bbox['bbox']['XG'],self.bbox['bbox']['YG']
@@ -99,9 +65,6 @@ class Background:
         else:
             tmpmask = np.full_like(obj.data['Y'],False,dtype=bool)
             return tmpmask
-    def _rootname(self):
-        tmp = fits.open(self.data['gfile'])[0].header['ROOTNAME']
-        return tmp
     def _trace(self):
         tmp = pd.read_csv(self.data['tfile'])
         mdydx = np.isfinite(tmp['DYDX'].values)
@@ -136,81 +99,14 @@ class Background:
     ##########
     ##########
     ##########
-    def loadinfo(self,savefolder):
-        print('Loading from {0}'.format(savefolder))
-        # load _data.json
-        filename = glob.glob('./{0}/*_data.json'.format(savefolder))[0]
-        with open(filename) as tmp:
-            tmpp = json.load(tmp)
-        data = copy.deepcopy(tmpp)
-        # load _bbox.json
-        filename = glob.glob('./{0}/*_bbox.json'.format(savefolder))[0]
-        with open(filename) as tmp:
-            tmpp = json.load(tmp)
-        bbox = copy.deepcopy(tmpp)
-        # construct arguments
-        key_data = ['objname','gfile','tfile','extnum','savefolder','overwrite','loadinfo',
-               'automask','rescale','sigclip']
-        key_bbox = ['padyup','padylow','halfdyup','halfdylow','padxleft','padxright']
-        args = {}
-        for i in key_data:
-            if i=='savefolder':
-                args[i] = './tmp_loading_session'
-                continue
-            if i=='overwrite':
-                args[i] = True
-                continue
-            if i=='loadinfo':
-                args[i] = False
-                continue
-            if i=='rescale':
-                args[i] = (data['rescale'],data['rescale_x1'],data['rescale_x2'],data['rescale_y'])
-                continue
-            if i=='sigclip':
-                args[i] = (data['sigclip'],data['sigclip_niter'],data['sigclip_sigma'])
-                continue
-            args[i] = data[i]
-        for i in key_bbox:
-            args[i] = bbox[i]
-        # instantiate Background()
-        obj = Background(**args)
-        # fix savefolder
-        obj.data['savefolder'] = savefolder
-        # fix .bkg.model
-        # load _coef.csv
-        filename = glob.glob('./{0}/*_coef.csv'.format(savefolder))[0]
-        tmp = pd.read_csv(filename,index_col=0)
-        shape = tmp.shape
-        norder = shape[0]-1
-        coef = {}
-        for i in range(shape[0]):
-            for j in range(shape[1]):
-                if np.isfinite(tmp.iloc[i,j]):
-                    coef[(i,j)] = tmp.iloc[i,j]
-        # load _maskfit.csv
-        filename = glob.glob('./{0}/*_maskfit.csv'.format(savefolder))[0]
-        tmp = pd.read_csv(filename,index_col=0)
-        maskfit = tmp.values.copy()
-        # load _pcov.csv
-        filename = glob.glob('./{0}/*_pcov.csv'.format(savefolder))[0]
-        tmp = pd.read_csv(filename,index_col=0)
-        pcov = tmp.values.copy()
-        obj.bkg.model['NORDER'] = norder
-        obj.bkg.model['COEF'] = coef
-        obj.bkg.model['MASKFIT'] = maskfit
-        obj.bkg.model['pcov'] = pcov
-        obj.bkg.compute()
-        return(obj)
-    ##########
-    ##########
-    ##########
-    def saveinfo(self):
-        savefolder = self.data['savefolder']
-        # save bkg sub as the same original name
-        string = './{1}/{0}'.format(self.data['gfile'].split('/')[-1],savefolder)
-        os.system('cp {0} ./{1}/'.format(self.data['gfile'],savefolder))
+    def save(self):
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        # save bkg sub as _sub.fits
+        string = './{1}/{0}_sub.fits'.format(saveprefix,savefolder)
+        os.system('cp {0} {1}'.format(self.data['gfile'][0],string))
         tmpp = fits.open(string)
-        tmppdata = tmpp[self.data['extnum']].data
+        tmppdata = tmpp[self.data['gfile'][1]].data
         bb0x,bb1x = self.bbox['bbox']['BBX']
         bb0y,bb1y = self.bbox['bbox']['BBY']
         y,yfit = self.bkg.data['Y'],self.bkg.model['YFIT']
@@ -227,46 +123,26 @@ class Background:
                 tmpp['BBX'] = self.bbox[j]['BBX']
                 tmpp['BBY'] = self.bbox[j]['BBY']
         tmppp = json.dumps(tmpp)
-        string = './{1}/{0}_bbox.json'.format(self.data['ROOTNAME'],savefolder)
+        string = './{1}/{0}_bkgbbox.json'.format(saveprefix,savefolder)
         f = open(string,'w')
         f.write(tmppp)
         f.close()
         print('Save {0}'.format(string))
-        # save only bkg as *_bkg.fits
-        string = './{1}/{0}'.format(self.data['gfile'].split('/')[-1].split('_')[0]+'_bkg.fits',savefolder)
-        os.system('cp {0} {1}'.format(self.data['gfile'],string))
+        # save only bkg replaced in the original as _bkg.fits
+        string = './{1}/{0}_bkg.fits'.format(saveprefix,savefolder)
+        os.system('cp {0} {1}'.format(self.data['gfile'][0],string))
         tmpp = fits.open(string)
-        tmppdata = tmpp[self.data['extnum']].data
+        tmppdata = tmpp[self.data['gfile'][1]].data
         bb0x,bb1x = self.bbox['bbox']['BBX']
         bb0y,bb1y = self.bbox['bbox']['BBY']
         yfit = self.bkg.model['YFIT']
         tmppdata[bb0y:bb1y,bb0x:bb1x] = yfit.copy()
         tmpp.writeto(string,overwrite=True)
         print('Save {0}'.format(string))
-        # save trace
-        ##### xyref
-        xref = np.full_like(self.trace['XG'],None,dtype=float)
-        yref = np.full_like(self.trace['XG'],None,dtype=float)
-        xref[0],yref[0] = self.trace['XREF'],self.trace['YREF']
-        ##### dydx, dldp
-        dydx = np.full_like(self.trace['XG'],None,dtype=float)
-        dldp = np.full_like(self.trace['XG'],None,dtype=float)
-        for ii,i in enumerate(self.trace['DYDX']):
-            dydx[ii] = i
-        for ii,i in enumerate(self.trace['DLDP']):
-            dldp[ii] = i
-        tmp = {'XREF':xref,'YREF':yref,'XG':self.trace['XG'],'YG':self.trace['YG'],'WW':self.trace['WW'],
-               'DYDX':dydx,'DLDP':dldp
-              }
-        string_trace = './{1}/{0}_trace.csv'.format(self.data['ROOTNAME'],savefolder)
-        pd.DataFrame(tmp).to_csv(string_trace,index=False)
-        print('Save {0}'.format(string_trace)) 
-        # save data
-        # with new trace location
+        # save self.data
         tmp = copy.deepcopy(self.data)
-        tmp['tfile'] = string_trace
         tmppp = json.dumps(tmp)
-        string = './{1}/{0}_data.json'.format(self.data['ROOTNAME'],savefolder)
+        string = './{1}/{0}_bkgdata.json'.format(saveprefix,savefolder)
         f = open(string,'w')
         f.write(tmppp)
         f.close()
@@ -277,29 +153,28 @@ class Background:
         tmp = np.full((norder+1,norder+1),fill_value=None,dtype=float)
         for i in coef:
             tmp[i[0],i[1]] = coef[i]
-        string = './{1}/{0}_coef.csv'.format(self.data['ROOTNAME'],savefolder)
+        string = './{1}/{0}_bkgcoef.csv'.format(saveprefix,savefolder)
         pd.DataFrame(tmp).to_csv(string,index=True)        
         print('Save {0}'.format(string)) 
         # save pcov
         pcov = self.bkg.model['pcov']
-        string = './{1}/{0}_pcov.csv'.format(self.data['ROOTNAME'],savefolder)
+        string = './{1}/{0}_bkgpcov.csv'.format(saveprefix,savefolder)
         pd.DataFrame(pcov).to_csv(string,index=True)        
         print('Save {0}'.format(string)) 
         # save maskfit
         maskfit = self.bkg.model['MASKFIT']
-        string = './{1}/{0}_maskfit.csv'.format(self.data['ROOTNAME'],savefolder)
+        string = './{1}/{0}_bkgmaskfit.csv'.format(saveprefix,savefolder)
         pd.DataFrame(maskfit).to_csv(string,index=True)        
         print('Save {0}'.format(string)) 
     ##########
     ##########
     ##########
-    def show_1d(self,
-                save=False,savename_prefix='plot',saveformat='eps',
+    def show_1d(self,save=False,
                 params={'figsize':(10,10),
                         'minmax':(5.,99.),
                         'cmap':'rainbow',
                         'fontsize':12,
-                        'tickperx':50,
+                        'xpertick':50,
                         'color':'red',
                         'marker':'x',
                         'ls':':',
@@ -307,8 +182,11 @@ class Background:
                         'annotate_level':0.
                        }
                ):
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        saveformat = self.container.data['plotformat']
         figsize = params['figsize']
-        tickperx = params['tickperx']
+        xpertick = params['xpertick']
         color = params['color']
         marker = params['marker']
         fontsize = params['fontsize']
@@ -317,7 +195,6 @@ class Background:
         ls = params['ls']
         minmax = params['minmax']
         cmap = params['cmap']
-        root = self.data['ROOTNAME']
         xref = self.trace['XREF']
         yref = self.trace['YREF']
         tmp = self.bkg.data['Y'] - self.bkg.model['YFIT']
@@ -328,7 +205,7 @@ class Background:
         fig = plt.figure(figsize=figsize)
         ax = fig.add_subplot(2,2,1)
         ax.imshow(tmp,origin='lower',cmap=cmap,vmin=vmin,vmax=vmax)
-        ax.set_title('{0}_bkgsub'.format(root),fontsize=fontsize)
+        ax.set_title('{0}_bkgsub'.format(saveprefix),fontsize=fontsize)
         ax.set_ylabel('pixY - {0}'.format(int(yref)),fontsize=fontsize)
         ax2 = fig.add_subplot(2,2,2,sharey=ax)
         tmpx = np.arange(len(tmppp))
@@ -342,7 +219,7 @@ class Background:
         ww = self.bbox['bbox']['WW']
         ax.plot(tmpx,tmpy,color=color,ls=ls)
         for i,ii in enumerate(tmpx):
-            if (i in {0,len(tmpx)-1}) or (np.mod(i,tickperx)==0):
+            if (i in {0,len(tmpx)-1}) or (np.mod(i,xpertick)==0):
                 label = '{0}A'.format(int(ww[i]))
                 ax.plot(tmpx[i],tmpy[i],color=color,marker=marker)
                 ax.annotate(label,(tmpx[i],tmpy[i]),
@@ -359,14 +236,13 @@ class Background:
         ax.set_xlabel('pixX - {0}'.format(int(xref)),fontsize=fontsize)
         fig.tight_layout()
         if save:
-            string = './{2}/{0}_1d.{1}'.format(savename_prefix,saveformat,self.data['savefolder'])
+            string = './{2}/{0}_1d.{1}'.format(saveprefix,saveformat,savefolder)
             fig.savefig(string,format=saveformat,bbox_inches='tight')
             print('Save {0}'.format(string))
     ##########
     ##########
     ##########
-    def show_bkg2dsub(self,
-                      save=False,savename_prefix='plot',saveformat='eps',
+    def show_bkg2dsub(self,save=False,
                       params={'figsize':(30,10),
                               'minmax_y':(10.,99.),
                               'minmax_ysub':(5.,99.),
@@ -374,6 +250,9 @@ class Background:
                               'fontsize':12
                              }
                      ):
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        saveformat = self.container.data['plotformat']
         figsize = params['figsize']
         minmax_y = params['minmax_y']
         minmax_ysub = params['minmax_ysub']
@@ -385,9 +264,9 @@ class Background:
         ax = fig.add_subplot(1,3,1)
         vmin,vmax = np.percentile(y,minmax_y[0]),np.percentile(y,minmax_y[1])
         ax.imshow(y,origin='lower',cmap=cmap,vmin=vmin,vmax=vmax)
-        ax.set_title(self.data['ROOTNAME'],fontsize=fontsize)
+        ax.set_title(saveprefix,fontsize=fontsize)
         ax = fig.add_subplot(1,3,2)
-        ax.imshow(yfit,origin='lower',cmap=cmap)
+        ax.imshow(yfit,origin='lower',cmap=cmap,vmin=vmin,vmax=vmax)
         ax.set_title('fit',fontsize=fontsize)
         ax = fig.add_subplot(1,3,3)
         vmin,vmax = np.percentile(ysub,minmax_ysub[0]),np.percentile(ysub,minmax_ysub[1])
@@ -395,11 +274,10 @@ class Background:
         ax.set_title('sub',fontsize=fontsize)
         fig.tight_layout()               
         if save:
-            string = './{2}/{0}_bkg2dsub.{1}'.format(savename_prefix,saveformat,self.data['savefolder'])
+            string = './{2}/{0}_bkg2dsub.{1}'.format(saveprefix,saveformat,savefolder)
             plt.savefig(string,format=saveformat,bbox_inches='tight')
             print('Save {0}'.format(string))
-    def show_bkg3d(self,
-                   save=False,savename_prefix='plot',saveformat='eps',
+    def show_bkg3d(self,save=False,
                    params={'figsize':(20,10),
                            'cmap':'rainbow',
                            'minmax':(5.,99.),
@@ -408,6 +286,9 @@ class Background:
                            'fontsize':12
                           }
                   ):
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        saveformat = self.container.data['plotformat']
         figsize = params['figsize']
         cmap = params['cmap']
         view_init_y = params['view_init_y']
@@ -424,7 +305,7 @@ class Background:
         ax = fig.add_subplot(2,1,1,projection='3d')
         ax.plot_surface(x1,x2,y,cmap=cmap,vmin=vmin,vmax=vmax)
         ax.view_init(*view_init_y)
-        ax.set_title(self.data['ROOTNAME'],fontsize=fontsize)
+        ax.set_title(saveprefix,fontsize=fontsize)
         ax = fig.add_subplot(2,1,2,projection='3d')
         tmpy = y.copy()
         tmpy[mask] = np.nan
@@ -433,14 +314,13 @@ class Background:
         ax.set_title('with MASKFIT',fontsize=fontsize)
         fig.tight_layout()               
         if save:
-            string = './{2}/{0}_bkg3d.{1}'.format(savename_prefix,saveformat,self.data['savefolder'])
+            string = './{2}/{0}_bkg3d.{1}'.format(saveprefix,saveformat,savefolder)
             plt.savefig(string,format=saveformat,bbox_inches='tight')
             print('Save {0}'.format(string))
-    def show_bbox2d(self,
-                    save=False,savename_prefix='plot',saveformat='eps',
+    def show_bbox2d(self,save=False,savesuffix=None,
                     params={'figsize':(10,10),
                             'minmax':(10.,80.),
-                            'tickperx':10,
+                            'xpertick':10,
                             'fontsize':12,
                             'rotation':30.,
                             'color':'r',
@@ -453,7 +333,12 @@ class Background:
                             'border_y':50
                            }
                    ):
-        tmpdata = fits.open(self.data['gfile'])[self.data['extnum']].data
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        saveformat = self.container.data['plotformat']
+        if savesuffix is None:
+            savesuffix = 'bbox2d'
+        tmpdata = fits.open(self.data['gfile'][0])[self.data['gfile'][1]].data
         minmax = params['minmax']
         cmap = params['cmap']
         figsize = params['figsize']
@@ -464,7 +349,7 @@ class Background:
         border_x = params['border_x']
         border_y = params['border_y']
         fontsize = params['fontsize']
-        tickperx = params['tickperx']
+        xpertick = params['xpertick']
         marker = params['marker']
         rotation = params['rotation']
         m = np.isfinite(tmpdata)
@@ -477,7 +362,7 @@ class Background:
         # plot trace with annotate for wavelength
         plt.plot(xg,yg,color=color,ls=ls,lw=lw,alpha=alpha)
         for i,ii in enumerate(xg):
-            if (i in {0,len(xg)-1}) or (np.mod(i,tickperx)==0):
+            if (i in {0,len(xg)-1}) or (np.mod(i,xpertick)==0):
                 label = '{0}A'.format(int(ww[i]))
                 plt.plot(xg[i],yg[i],color=color,marker=marker)
                 plt.annotate(label,(xg[i],yg[i]),
@@ -505,10 +390,10 @@ class Background:
         # set frame
         plt.xlim(bb0x-border_x,bb1x+border_x)
         plt.ylim(bb0y-border_y,bb1y+border_y)
-        plt.title(self.data['ROOTNAME'],fontsize=fontsize) 
+        plt.title(saveprefix,fontsize=fontsize) 
         plt.tight_layout()
         if save:
-            string = './{2}/{0}_bbox2d.{1}'.format(savename_prefix,saveformat,self.data['savefolder'])
+            string = './{2}/{0}_{3}.{1}'.format(saveprefix,saveformat,savefolder,savesuffix)
             plt.savefig(string,format=saveformat,bbox_inches='tight')
             print('Save {0}'.format(string))
             
\ No newline at end of file
